@@ -1,6 +1,6 @@
-"""
-Pymodbus Server With Updating Thread
---------------------------------------------------------------------------
+# pylint: disable=differing-param-doc,missing-any-param-doc
+"""Pymodbus Server With Updating Thread.
+
 This is an example of having a background thread updating the
 context in an SQLite4 database while the server is operating.
 
@@ -13,28 +13,26 @@ This can also be done with a python thread::
     thread = Thread(target=updating_writer, args=(context,))
     thread.start()
 """
+import asyncio
+import logging
+import random
+
+from pymodbus.datastore import ModbusSequentialDataBlock, ModbusServerContext
+from pymodbus.datastore.database import SqlSlaveContext
+from pymodbus.device import ModbusDeviceIdentification
+from pymodbus.server import StartAsyncTcpServer
+
 # --------------------------------------------------------------------------- #
 # import the modbus libraries we need
 # --------------------------------------------------------------------------- #
 from pymodbus.version import version
-from pymodbus.server.asynchronous import StartTcpServer
-from pymodbus.device import ModbusDeviceIdentification
-from pymodbus.datastore import ModbusSequentialDataBlock
-from pymodbus.datastore import ModbusServerContext
-from pymodbus.datastore.database import SqlSlaveContext
-from pymodbus.transaction import ModbusRtuFramer, ModbusAsciiFramer
-import random
 
-# --------------------------------------------------------------------------- #
-# import the twisted libraries we need
-# --------------------------------------------------------------------------- #
-from twisted.internet.task import LoopingCall
+
+# from pymodbus.transaction import ModbusRtuFramer, ModbusAsciiFramer
 
 # --------------------------------------------------------------------------- #
 # configure the service logging
 # --------------------------------------------------------------------------- #
-import logging
-logging.basicConfig()
 log = logging.getLogger()
 log.setLevel(logging.DEBUG)
 
@@ -43,37 +41,40 @@ log.setLevel(logging.DEBUG)
 # --------------------------------------------------------------------------- #
 
 
-def updating_writer(a):
-    """ A worker process that runs every so often and
-    updates live values of the context which resides in an SQLite3 database.
+def updating_writer(parm1):
+    """Run every so often,
+
+    and updates live values of the context which resides in an SQLite3 database.
     It should be noted that there is a race condition for the update.
     :param arguments: The input arguments to the call
     """
     log.debug("Updating the database context")
-    context  = a[0]
-    readfunction = 0x03 # read holding registers
+    context = parm1[0]
+    readfunction = 0x03  # read holding registers
     writefunction = 0x10
-    slave_id = 0x01 # slave address
+    slave_id = 0x01  # slave address
     count = 50
 
     # import pdb; pdb.set_trace()
 
-    rand_value = random.randint(0, 9999)
-    rand_addr = random.randint(0, 65000)
-    log.debug("Writing to datastore: {}, {}".format(rand_addr, rand_value))
+    rand_value = random.randint(0, 9999)  # nosec
+    rand_addr = random.randint(0, 65000)  # nosec
+    txt = f"Writing to datastore: {rand_addr}, {rand_value}"
+    log.debug(txt)
     # import pdb; pdb.set_trace()
-    context[slave_id].setValues(writefunction, rand_addr, [rand_value],
-                                update=False)
+    context[slave_id].setValues(writefunction, rand_addr, [rand_value], update=False)
     values = context[slave_id].getValues(readfunction, rand_addr, count)
-    log.debug("Values from datastore: " + str(values))
+    txt = f"Values from datastore: {values}"
+    log.debug(txt)
 
 
-def run_dbstore_update_server():
+async def run_dbstore_update_server():
+    """Run dbstore update server."""
     # ----------------------------------------------------------------------- #
     # initialize your data store
     # ----------------------------------------------------------------------- #
 
-    block = ModbusSequentialDataBlock(0x00, [0] * 0xff)
+    block = ModbusSequentialDataBlock(0x00, [0] * 0xFF)
     store = SqlSlaveContext(block)
 
     context = ModbusServerContext(slaves={1: store}, single=False)
@@ -81,25 +82,26 @@ def run_dbstore_update_server():
     # ----------------------------------------------------------------------- #
     # initialize the server information
     # ----------------------------------------------------------------------- #
-    identity = ModbusDeviceIdentification()
-    identity.VendorName = 'pymodbus'
-    identity.ProductCode = 'PM'
-    identity.VendorUrl = 'http://github.com/riptideio/pymodbus/'
-    identity.ProductName = 'pymodbus Server'
-    identity.ModelName = 'pymodbus Server'
-    identity.MajorMinorRevision = version.short()
+    identity = ModbusDeviceIdentification(
+        info_name={
+            "VendorName": "pymodbus",
+            "ProductCode": "PM",
+            "VendorUrl": "https://github.com/riptideio/pymodbus/",
+            "ProductName": "pymodbus Server",
+            "ModelName": "pymodbus Server",
+            "MajorMinorRevision": version.short(),
+        }
+    )
 
     # ----------------------------------------------------------------------- #
     # run the server you want
     # ----------------------------------------------------------------------- #
     time = 5  # 5 seconds delay
-    loop = LoopingCall(f=updating_writer, a=(context,))
+    loop = asyncio.get_event_loop()
     loop.start(time, now=False)  # initially delay by time
     loop.stop()
-    StartTcpServer(context, identity=identity, address=("", 5020))
+    await StartAsyncTcpServer(context, identity=identity, address=("", 5020))
 
 
 if __name__ == "__main__":
-    run_dbstore_update_server()
-
-
+    asyncio.run(run_dbstore_update_server())
