@@ -1,120 +1,108 @@
-"""
-Pymodbus Exceptions
---------------------
+"""Pymodbus Exceptions.
 
 Custom exceptions to be used in the Modbus code.
 """
 
 
 class ModbusException(Exception):
-    """ Base modbus exception """
+    """Base modbus exception."""
 
     def __init__(self, string):
-        """ Initialize the exception
+        """Initialize the exception.
+
         :param string: The message to append to the error
         """
         self.string = string
+        super().__init__()
 
     def __str__(self):
-        return 'Modbus Error: %s' % self.string
+        """Return string representation."""
+        return f"Modbus Error: {self.string}"
 
-    def isError(self):
+    def isError(self):  # pylint: disable=invalid-name
         """Error"""
         return True
 
 
 class ModbusIOException(ModbusException):
-    """ Error resulting from data i/o """
+    """Error resulting from data i/o."""
 
     def __init__(self, string="", function_code=None):
-        """ Initialize the exception
+        """Initialize the exception.
+
         :param string: The message to append to the error
         """
         self.fcode = function_code
-        self.message = "[Input/Output] %s" % string
+        self.message = f"[Input/Output] {string}"
         ModbusException.__init__(self, self.message)
 
 
 class ParameterException(ModbusException):
-    """ Error resulting from invalid parameter """
+    """Error resulting from invalid parameter."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
+        """Initialize the exception.
 
         :param string: The message to append to the error
         """
-        message = "[Invalid Parameter] %s" % string
+        message = f"[Invalid Parameter] {string}"
         ModbusException.__init__(self, message)
 
 
 class NoSuchSlaveException(ModbusException):
-    """ Error resulting from making a request to a slave
-    that does not exist """
+    """Error resulting from making a request to a slave that does not exist."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
+        """Initialize the exception.
 
         :param string: The message to append to the error
         """
-        message = "[No Such Slave] %s" % string
+        message = f"[No Such Slave] {string}"
         ModbusException.__init__(self, message)
 
 
 class NotImplementedException(ModbusException):
-    """ Error resulting from not implemented function """
+    """Error resulting from not implemented function."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
+        """Initialize the exception.
+
         :param string: The message to append to the error
         """
-        message = "[Not Implemented] %s" % string
+        message = f"[Not Implemented] {string}"
         ModbusException.__init__(self, message)
 
 
 class ConnectionException(ModbusException):
-    """ Error resulting from a bad connection """
+    """Error resulting from a bad connection."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
+        """Initialize the exception.
 
         :param string: The message to append to the error
         """
-        message = "[Connection] %s" % string
+        message = f"[Connection] {string}"
         ModbusException.__init__(self, message)
 
 
 class InvalidMessageReceivedException(ModbusException):
-    """
-    Error resulting from invalid response received or decoded
-    """
+    """Error resulting from invalid response received or decoded."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
+        """Initialize the exception.
 
         :param string: The message to append to the error
         """
-        message = "[Invalid Message] %s" % string
+        message = f"[Invalid Message] {string}"
         ModbusException.__init__(self, message)
 
 
 class MessageRegisterException(ModbusException):
-    """
-    Error resulting from failing to register a custom message request/response
-    """
-    def __init__(self, string=""):
-        message = '[Error registering message] %s' % string
-        ModbusException.__init__(self, message)
-
-
-class TimeOutException(ModbusException):
-    """ Error resulting from modbus response timeout """
+    """Error resulting from failing to register a custom message request/response."""
 
     def __init__(self, string=""):
-        """ Initialize the exception
-
-        :param string: The message to append to the error
-        """
-        message = "[Timeout] %s" % string
+        """Initialize."""
+        message = f"[Error registering message] {string}"
         ModbusException.__init__(self, message)
 
 
@@ -122,9 +110,12 @@ class TimeOutException(ModbusException):
 # Exported symbols
 # --------------------------------------------------------------------------- #
 __all__ = [
-    "ModbusException", "ModbusIOException",
-    "ParameterException", "NotImplementedException",
-    "ConnectionException", "NoSuchSlaveException",
+    "ModbusException",
+    "ModbusIOException",
+    "ParameterException",
+    "NotImplementedException",
+    "ConnectionException",
+    "NoSuchSlaveException",
     "InvalidMessageReceivedException",
-    "MessageRegisterException", "TimeOutException"
+    "MessageRegisterException",
 ]
