@@ -1,18 +1,15 @@
-"""
-Modbus Utilities
------------------
+"""Modbus Utilities.
 
 A collection of utilities for packing data, unpacking
 data computing checksums, and decode checksums.
 """
-from pymodbus.compat import int2byte, byte2int, IS_PYTHON3
-from six import string_types
+# pylint: disable=missing-type-doc
+import struct
 
 
-class ModbusTransactionState(object):
-    """
-    Modbus Client States
-    """
+class ModbusTransactionState:  # pylint: disable=too-few-public-methods
+    """Modbus Client States."""
+
     IDLE = 0
     SENDING = 1
     WAITING_FOR_REPLY = 2
@@ -25,6 +22,7 @@ class ModbusTransactionState(object):
 
     @classmethod
     def to_string(cls, state):
+        """Convert to string."""
         states = {
             ModbusTransactionState.IDLE: "IDLE",
             ModbusTransactionState.SENDING: "SENDING",
@@ -42,10 +40,9 @@ class ModbusTransactionState(object):
 # Helpers
 # --------------------------------------------------------------------------- #
 
+
 def default(value):
-    """
-    Given a python object, return the default value
-    of that object.
+    """Return the default value of object.
 
     :param value: The value to get the default of
     :returns: The default value
@@ -54,7 +51,8 @@ def default(value):
 
 
 def dict_property(store, index):
-    """ Helper to create class properties from a dictionary.
+    """Create class properties from a dictionary.
+
     Basically this allows you to remove a lot of possible
     boilerplate code.
 
@@ -62,16 +60,33 @@ def dict_property(store, index):
     :param index: The index into the store to close over
     :returns: An initialized property set
     """
-    if hasattr(store, '__call__'):
-        getter = lambda self: store(self)[index]
-        setter = lambda self, value: store(self).__setitem__(index, value)
+    if hasattr(store, "__call__"):
+        getter = lambda self: store(  # pylint: disable=unnecessary-lambda-assignment
+            self
+        )[index]
+        setter = lambda self, value: store(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+            self
+        ).__setitem__(
+            index, value
+        )
     elif isinstance(store, str):
-        getter = lambda self: self.__getattribute__(store)[index]
-        setter = lambda self, value: self.__getattribute__(store).__setitem__(
-            index, value)
+        getter = lambda self: self.__getattribute__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+            store
+        )[
+            index
+        ]
+        setter = lambda self, value: self.__getattribute__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+            store
+        ).__setitem__(
+            index, value
+        )
     else:
-        getter = lambda self: store[index]
-        setter = lambda self, value: store.__setitem__(index, value)
+        getter = lambda self: store[  # pylint: disable=unnecessary-lambda-assignment
+            index
+        ]
+        setter = lambda self, value: store.__setitem__(  # pylint: disable=unnecessary-dunder-call,unnecessary-lambda-assignment
+            index, value
+        )
 
     return property(getter, setter)
 
@@ -80,7 +95,7 @@ def dict_property(store, index):
 # Bit packing functions
 # --------------------------------------------------------------------------- #
 def pack_bitstring(bits):
-    """ Creates a string out of an array of bits
+    """Create a string out of an array of bits.
 
     :param bits: A bit array
 
@@ -89,60 +104,61 @@ def pack_bitstring(bits):
         bits   = [False, True, False, True]
         result = pack_bitstring(bits)
     """
-    ret = b''
+    ret = b""
     i = packed = 0
     for bit in bits:
         if bit:
             packed += 128
         i += 1
         if i == 8:
-            ret += int2byte(packed)
+            ret += struct.pack(">B", packed)
             i = packed = 0
         else:
             packed >>= 1
     if 0 < i < 8:
-        packed >>= (7 - i)
-        ret += int2byte(packed)
+        packed >>= 7 - i
+        ret += struct.pack(">B", packed)
     return ret
 
 
 def unpack_bitstring(string):
-    """ Creates bit array out of a string
+    """Create bit array out of a string.
 
     :param string: The modbus data packet to decode
 
     example::
 
-        bytes  = 'bytes to decode'
+        bytes  = "bytes to decode"
         result = unpack_bitstring(bytes)
     """
     byte_count = len(string)
     bits = []
     for byte in range(byte_count):
-        if IS_PYTHON3:
-            value = byte2int(int(string[byte]))
-        else:
-            value = byte2int(string[byte])
+        value = int(int(string[byte]))
         for _ in range(8):
             bits.append((value & 1) == 1)
             value >>= 1
     return bits
 
 
-def make_byte_string(s):
-    """
-    Returns byte string from a given string, python3 specific fix
-    :param s:
+def make_byte_string(byte_string):
+    """Return byte string from a given string, python3 specific fix.
+
+    :param byte_string:
     :return:
     """
-    if IS_PYTHON3 and isinstance(s, string_types):
-        s = s.encode()
-    return s
+    if isinstance(byte_string, str):
+        byte_string = byte_string.encode()
+    return byte_string
+
+
 # --------------------------------------------------------------------------- #
 # Error Detection Functions
 # --------------------------------------------------------------------------- #
+
+
 def __generate_crc16_table():
-    """ Generates a crc16 lookup table
+    """Generate a crc16 lookup table.
 
     .. note:: This will only be generated once
     """
@@ -151,36 +167,39 @@ def __generate_crc16_table():
         crc = 0x0000
         for _ in range(8):
             if (byte ^ crc) & 0x0001:
-                crc = (crc >> 1) ^ 0xa001
-            else: crc >>= 1
+                crc = (crc >> 1) ^ 0xA001
+            else:
+                crc >>= 1
             byte >>= 1
         result.append(crc)
     return result
 
+
 __crc16_table = __generate_crc16_table()
 
 
-def computeCRC(data):
-    """ Computes a crc16 on the passed in string. For modbus,
-    this is only used on the binary serial protocols (in this
+def computeCRC(data):  # pylint: disable=invalid-name
+    """Compute a crc16 on the passed in string.
+
+    For modbus, this is only used on the binary serial protocols (in this
     case RTU).
 
-    The difference between modbus's crc16 and a normal crc16
+    The difference between modbus"s crc16 and a normal crc16
     is that modbus starts the crc value out at 0xffff.
 
     :param data: The data to create a crc16 of
     :returns: The calculated CRC
     """
-    crc = 0xffff
-    for a in data:
-        idx = __crc16_table[(crc ^ byte2int(a)) & 0xff]
-        crc = ((crc >> 8) & 0xff) ^ idx
-    swapped = ((crc << 8) & 0xff00) | ((crc >> 8) & 0x00ff)
+    crc = 0xFFFF
+    for data_byte in data:
+        idx = __crc16_table[(crc ^ int(data_byte)) & 0xFF]
+        crc = ((crc >> 8) & 0xFF) ^ idx
+    swapped = ((crc << 8) & 0xFF00) | ((crc >> 8) & 0x00FF)
     return swapped
 
 
-def checkCRC(data, check):
-    """ Checks if the data matches the passed in CRC
+def checkCRC(data, check):  # pylint: disable=invalid-name
+    """Check if the data matches the passed in CRC.
 
     :param data: The data to create a crc16 of
     :param check: The CRC to validate
@@ -189,9 +208,10 @@ def checkCRC(data, check):
     return computeCRC(data) == check
 
 
-def computeLRC(data):
-    """ Used to compute the longitudinal redundancy check
-    against a string. This is only used on the serial ASCII
+def computeLRC(data):  # pylint: disable=invalid-name
+    """Use to compute the longitudinal redundancy check against a string.
+
+    This is only used on the serial ASCII
     modbus protocol. A full description of this implementation
     can be found in appendex B of the serial line modbus description.
 
@@ -199,13 +219,13 @@ def computeLRC(data):
     :returns: The calculated LRC
 
     """
-    lrc = sum(byte2int(a) for a in data) & 0xff
-    lrc = (lrc ^ 0xff) + 1
-    return lrc & 0xff
+    lrc = sum(int(a) for a in data) & 0xFF
+    lrc = (lrc ^ 0xFF) + 1
+    return lrc & 0xFF
 
 
-def checkLRC(data, check):
-    """ Checks if the passed in data matches the LRC
+def checkLRC(data, check):  # pylint: disable=invalid-name
+    """Check if the passed in data matches the LRC.
 
     :param data: The data to calculate
     :param check: The LRC to validate
@@ -214,8 +234,8 @@ def checkLRC(data, check):
     return computeLRC(data) == check
 
 
-def rtuFrameSize(data, byte_count_pos):
-    """ Calculates the size of the frame based on the byte count.
+def rtuFrameSize(data, byte_count_pos):  # pylint: disable=invalid-name
+    """Calculate the size of the frame based on the byte count.
 
     :param data: The buffer containing the frame.
     :param byte_count_pos: The index of the byte count in the buffer.
@@ -234,25 +254,30 @@ def rtuFrameSize(data, byte_count_pos):
     field, and finally increment the sum by three (one byte for the
     byte count field, two for the CRC).
     """
-    return byte2int(data[byte_count_pos]) + byte_count_pos + 3
+    return int(data[byte_count_pos]) + byte_count_pos + 3
 
 
 def hexlify_packets(packet):
-    """
-    Returns hex representation of bytestring received
+    """Return hex representation of bytestring received.
+
     :param packet:
     :return:
     """
     if not packet:
-        return ''
-    if IS_PYTHON3:
-        return " ".join([hex(byte2int(x)) for x in packet])
-    else:
-        return u" ".join([hex(byte2int(x)) for x in packet])
+        return ""
+    return " ".join([hex(int(x)) for x in packet])
+
+
 # --------------------------------------------------------------------------- #
 # Exported symbols
 # --------------------------------------------------------------------------- #
 __all__ = [
-    'pack_bitstring', 'unpack_bitstring', 'default',
-    'computeCRC', 'checkCRC', 'computeLRC', 'checkLRC', 'rtuFrameSize'
+    "pack_bitstring",
+    "unpack_bitstring",
+    "default",
+    "computeCRC",
+    "checkCRC",
+    "computeLRC",
+    "checkLRC",
+    "rtuFrameSize",
 ]
