@@ -1,16 +1,14 @@
-"""
-Handle the version information here; you should only have to
-change the version tuple.
+"""Handle the version information here.
 
-Since we are using twisted's version class, we can also query
-the svn version as well using the local .entries file.
+you should only have to change the version tuple.
 """
 
 
-class Version(object):
+class Version:
+    """Manage version."""
 
     def __init__(self, package, major, minor, micro, pre=None):
-        """
+        """Initialize.
 
         :param package: Name of the package that this is a version of.
         :param major: The major version number.
@@ -25,28 +23,30 @@ class Version(object):
         self.pre = pre
 
     def short(self):
-        """ Return a string in canonical short version format
-        <major>.<minor>.<micro>.<pre>
-        """
+        """Return a string in canonical short version format: <major>.<minor>.<micro>.<pre>."""
+        pre = ""
         if self.pre:
-            return '%d.%d.%d.%s' % (self.major, self.minor, self.micro, self.pre)
-        else:
-            return '%d.%d.%d' % (self.major, self.minor, self.micro)
+            pre = f".{self.pre}"
+        return f"{self.major}.{self.minor}.{self.micro}{pre}"
 
     def __str__(self):
-        """ Returns a string representation of the object
+        """Return a string representation of the object.
 
         :returns: A string representation of this object
         """
-        return '[%s, version %s]' % (self.package, self.short())
+        return f"[{self.package}, version {self.short()}]"
 
 
-version = Version('pymodbus', 2, 5, 3)
+version = Version("pymodbus", 3, 0, 0, "rc1")
+version.__name__ = (  # fix epydoc error # pylint: disable=attribute-defined-outside-init
+    "pymodbus"
+)
 
-version.__name__ = 'pymodbus'  # fix epydoc error
 
 # --------------------------------------------------------------------------- #
 # Exported symbols
 # --------------------------------------------------------------------------- #
 
-__all__ = ["version"]
+__all__ = [
+    "version"
+]
